library zub_tools;

// /// A Calculator.
// class Calculator {
//   /// Returns [value] plus 1.
//   int addOne(int value) => value + 1;
// }

import 'dart:convert';
import 'dart:io' show Platform;
import 'dart:math';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

enum ScreenType { material, cupertino, builder }

mixin Utils {
/*
  //Singleton instance
  static Utils instance = Utils._instance();
  Utils._instance();

  //Singleton constructor
  factory Utils() {
    return instance;
  }
*/

  static String assetsImage(String name, {String path = 'assets/images'}) {
    return '$path/$name';
  }

  static String assetsIcon(String name, {String path = 'assets/icons'}) {
    return '$path/$name';
  }

  static void hideKeyboard(BuildContext context) {
//    FocusScope.of(context).requestFocus(new FocusNode());
    FocusScope.of(context).unfocus();
  }

  @Deprecated('Use FocusNode().requestFocus()')
  static void requestFocus(BuildContext context, FocusNode focusNode) {
    FocusScope.of(context).requestFocus(focusNode);
  }

  static void myLog(String message, {isJsonString = false}) {
    if (isJsonString) {
      final jsonObject = json.decode(message);
      final jsonFormattedString =
      JsonEncoder.withIndent('  ').convert(jsonObject);
      print(
          '<--Zubair Yasin--><${DateTime.now().toString()}>\n$jsonFormattedString');
    } else {
      print('<--Zubair Yasin--><${DateTime.now().toString()}>$message');
    }
  }

  static Future hideStatusBar() async {
    await SystemChrome.setEnabledSystemUIOverlays([]);
  }

  static Future showStatusBar() async {
    await SystemChrome.setEnabledSystemUIOverlays(SystemUiOverlay.values);

//    SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
//        systemNavigationBarColor: Colors.amber,
//        statusBarBrightness: Brightness.dark,
//        statusBarColor: Colors.deepPurple));
  }

  static void showRouteDialog(BuildContext context, Widget child) {
    Route route = PageRouteBuilder(
      opaque: false,
      pageBuilder: (_, __, ___) {
        return child;
      },
    );

    Navigator.of(context).push(route);
  }

  /*Use MediaQuery.of(context) as parameter object*/
  static bool isTablet(MediaQueryData query) {
    if (kIsWeb) {
      return true;
    }

    if (Platform.isWindows || Platform.isMacOS) {
      return false;
    }

    Size size = query.size;
    double diagonal =
    sqrt((size.width * size.width) + (size.height * size.height));
    bool isTablet = diagonal > 1100.0;
    return isTablet;
  }

  static Future<T> newScreen<T>(
      BuildContext context,
      Widget child, {
        ScreenType screenType = ScreenType.cupertino,
        bool backgroundLayer,
        bool replace = false,
      }) {
    myLog(
        'nextScreen(Type:${screenType.index}): ${child.runtimeType.toString()}');

    ///[backgroundLayer] (opaque) is used only in [ScreenType.builder]
    if (backgroundLayer != null) {
      screenType = ScreenType.builder;
    }

    Route<T> route;
    switch (screenType) {
      case ScreenType.cupertino:
        route = CupertinoPageRoute(
          settings: RouteSettings(
            name: child.runtimeType.toString(),
          ),
          builder: (_) {
            return child;
          },
        );
        break;
      case ScreenType.material:
        route = MaterialPageRoute(
          settings: RouteSettings(
            name: child.runtimeType.toString(),
          ),
          builder: (_) {
            return child;
          },
        );
        break;
      case ScreenType.builder:
        route = PageRouteBuilder(
          opaque: backgroundLayer ?? true,
          settings: RouteSettings(
            name: child.runtimeType.toString(),
          ),
          pageBuilder: (_, __, ___) {
            return child;
          },
        );
        break;
    }

    Future<T> t;
    switch (replace) {
      case true:
        t = Navigator.of(context).pushReplacement(route);
        break;
      case false:
        t = Navigator.of(context).push(route);
        break;
    }

    return t;
  }

  static void goBack<T>(BuildContext context, [T t]) {
    Navigator.of(context).pop(t);
  }

  //This method will be called after build() method.
  //If you place this method in initState() then it will be called only once.
  //If you place this method in build() then it will be called every time
  //after build call.
  static void afterBuild(VoidCallback fn) {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      fn();
    });
  }

}

class HexColor extends Color {
  static int getColorFromHex(String hexColor) {
    hexColor = hexColor.toUpperCase().replaceAll("#", "");
    if (hexColor.length == 6) {
      hexColor = "ff" + hexColor;
    }
    return int.parse(hexColor, radix: 16);
  }

  HexColor(final String hexColor) : super(getColorFromHex(hexColor));
}

//typedef WidgetBuilder = Widget Function(BuildContext context);
//
//typedef MyFunction = bool Function(BuildContext context);
